var express = require('express');
var app = express();
var fs = require('fs');
var Promise = require('bluebird');
var Request = require("request");
var JSPath = require("jspath");
var pdf = require('html-pdf');
var bodyParser = require("body-parser");
var _ = require('underscore');
var moment = require('moment');
var mongo = require('mongodb');
var logger = require('logger').createLogger('development.log');

app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));


/** For getting the Sorted value of date with minimum and maximum value of date
 *  req: Request for getting the json of shuffled date
 *  res: Response getting the sorted value of date as well as timestamp 
 *  sample Json: {
	"date" : ["03-01-2019","03-10-2019","03-04-2019","03-02-2019","03-09-2019"]
     }
 */
exports.sortDate = function (req, resp) {
    var jsonObj = [];
    var sortdate = {
        "date": []
    };
    var date = req.body.date;
    for (var i in date) {
        jsonObj.push({
            "date": date[i]
        })
    }
    var latestDate = _.sortBy(jsonObj, function (o) { return new moment(o.date); }).reverse();
    for (var i = 0; i < latestDate.length; i++) {
        sortdate.date.push(latestDate[i].date);
    }
    //Max Date value apped in sortdate
    sortdate.maxdate = latestDate[0].date;
    //Min Date value append in sortdate 
    sortdate.mindate = latestDate[latestDate.length - 1].date;
    resp.send(200, sortdate);
};

/** For getting the Sorted value of timestamp with minimum and maximum value of timestamp
 *  req: Request for getting the json of shuffled timestamp
 *  res: Response getting the sorted value of timestamp as well as timestamp 
 *  sample Json : {
   "timestamp" : ["2019-03-02T17:50:36.024Z","2019-03-03T19:50:36.024Z","2019-03-04T17:50:36.024Z","2019-03-01T17:50:36.024Z"]
    }
 */
exports.sortTimeStamp = function (req, resp) {
    var jsonObj = [];
    var sortdate = {
        "timestamp": []
    };
    var timeStamp = req.body.timestamp;
    for (var i in timeStamp) {
        jsonObj.push({
            "timeStamp": timeStamp[i]
        })
    }
    var latestTime = _.sortBy(jsonObj, function (o) { return new moment(o.timeStamp); }).reverse();
    for (var i = 0; i < latestTime.length; i++) {
        sortdate.timestamp.push(latestTime[i].timeStamp);
    }
    sortdate.maxtimestamp = latestTime[0].timeStamp;
    sortdate.mintimestamp = latestTime[latestTime.length - 1].timeStamp;
    resp.send(200, sortdate);
};


/** Function for getting the filtered value from an array 
 *  Req: array, Key in which the filter is applied, Value For checking the condition
 *  Response: Filtered Array JSON 
 * {
    "arr": [
        {
            "code_id": "1",
            "code_name": "code 1"
        },
        {
            "code_id": "2",
            "code_name": "code889"
        }
    ],
    "key": "code_id",
    "value": "2"
}
 */

exports.filterArray = function (req, res) {
    var getFilteredCodes = (array, key, value) => array.filter(x => x[key] == value);
    var FilteredCodes = getFilteredCodes(req.body.arr, req.body.key, req.body.value);
    res.send(200, FilteredCodes); 
};

/** array  Sorting ascending and descending order. 
 * req : send the array in request.
 * res : send the sorted array ressponse.
*/
exports.upperSort = function(req, res){
   arr = req.body.arr;
   arr.sort();
   var jsonObj = {
       "sortArr" : arr,
       "sort" : "Minimum to maximum",
       "order" : "Ascending"
   }
   res.send(200, jsonObj);
};

/** array  Sorting ascending and descending order. 
 * req : send the array in request.
 * res : send the sorted array ressponse.
*/
exports.lowerSort = function(req, res){
    arr = req.body.arr;
    arr.sort();
    arr.reverse();
    var jsonObj = {
        "sortArr" : arr,
        "sort" : "Maximum to minimum",
        "order": "Descending"
    }
    res.send(200, jsonObj);
 };
 
 /** array  Sorting ascending and descending order Based on the Key name. 
 * req : send the array in request.
 * Sample Array JSON {
    "arr": [
      { "ID": 135, "Name": "Fargo Chan", "Address": "34, Baker Street" },
      { "ID": 432, "Name": "Aaron Luke", "Address": "BLDG 1, J Street" },
      { "ID": 252, "Name": "Dilip Singh", "Address": "Hotel J, SE" }
    ],
    "key": "ID" // Key value Must Be changed According to the JSON Value Sorting must be done on the basis of Key Value
}
 * res : send the sorted array ressponse.
*/
exports.jsonArraySort = function(req, res){
    // Function for sorting the json Array Based on the key
    var arr = req.body.arr;
    var key = req.body.key;
    //Function For Sorting of Array Based on the Key Name
    function SortByName(x,y) {
        return ((x[key] == y[key]) ? 0 : ((x[key] > y[key]) ? 1 : -1 ));
      }
      //sort Function Based on the name of key
      arr.sort(SortByName);
      //JSONobj For Sending the response
      var jsonObj = {
          "sortArr" : arr
      }
      res.send(200, jsonObj);
};


app.listen(8080, function () {
    console.log("listening on 8080");
});